<?php

///////////////////////////////////////////////////////////////////////////////
/**
 * The SecureSession class helps protect against Cross Site Request Forgery (CSRF).
 * In addition to the cookie session ID, a token is transmitted to the client as well.
 * The token must be included in every request from the client back to the server as 
 * a GET, POST or SOAP parameter, but NOT as a COOKIE.
 * 
 * @author Per Egil Roksvaag
 */

class SecureSession
{
	
	///////////////////////////////////////////////////////////////////////////
	/**
	 * The number of seconds before the session expires.
	 * @property int
	 */
	
	protected static $lifetime;
	
	///////////////////////////////////////////////////////////////////////////
	/**
	 * Contains the $_SERVER field where the remote user id is stored.
	 * @property string
	 */
	
	protected static $userfield;
	
	///////////////////////////////////////////////////////////////////////////
	/**
	 * Starts the secure session - You MUST call this function before using 
	 * any of the other methods.
	 * @param int $lifetime - The number of seconds before the session expires.
	 * @param string $name - Name of the session cookie
	 * @return void
	 */
	
	public static function init($lifetime = 3600, $path = "/", $name = "SecureSession")
	{
		global $oe_https,$oe_cookie_httponly;
		self::$lifetime = $lifetime;
		self::$userfield = array_key_exists("AUTH_USER", $_SERVER) ? "AUTH_USER" : "REMOTE_ADDR";
		ini_set("session.gc_maxlifetime", $lifetime);
		ini_set("session.referer_check", $_SERVER["SERVER_NAME"]);
		session_name($name);
		if ($oe_cookie_httponly)
		{
			session_set_cookie_params(0, $path, null, $oe_https, true);
		}
		else
		{
			session_set_cookie_params(0, $path, null, $oe_https);
		}
		session_start();
		header("Pragma:"); // IE Bug Workaround
	}
	
	///////////////////////////////////////////////////////////////////////////
	/**
	 * Opens a new secure session and returns a token for verifying.   
	 * @return string - token to send to the client.
	 */
	
	public static function open()
	{
		if(count($_SESSION))
		{
			session_regenerate_id(true);
		}
		
		$_SESSION["time"] = time();
		$_SESSION["user"] = $_SERVER[self::$userfield];
		$_SESSION["token"] = md5(uniqid(rand(100000000, 999999999), true));
		return $_SESSION["token"];
	}

	///////////////////////////////////////////////////////////////////////////
	/**
	 * Verfies that the token belongs to the current session and user.  
	 * @param string $token - token sent with the request.
	 * @return void
	 */
	
	public static function verify($token)
	{
		$bTime = isset($_SESSION["time"]) && (time() - $_SESSION["time"] < self::$lifetime);
		$bUser = isset($_SESSION["user"]) && ($_SESSION["user"] == $_SERVER[self::$userfield]);
		$bToken = isset($_SESSION["token"]) && ($_SESSION["token"] == $token);
		if($bTime && $bUser && $bToken)
		{
			$_SESSION["time"] = time();
		}
		else
		{
			session_regenerate_id(true);
			session_unset();
			session_destroy();
			die("Permission denied");
		}
	}

	///////////////////////////////////////////////////////////////////////////
	/**
	 * Close the current secure session.
	 * @return bool
	 */
	
	public static function close()
	{
		return session_regenerate_id(true);
	}

	///////////////////////////////////////////////////////////////////////////
}

?>